# -*- coding: utf-8 -*-
from PyQt4.QtCore import QPoint
from PyQt4.QtGui import QColor

from qgis.core import QgsDistanceArea
from qgis.gui import QgsMapTool, QgsRubberBand

from dialogos.medir_distancia import DialogoMedirDistancia

class MedirDistanciaMapTool( QgsMapTool ):
    """
        Herramienta Medir Distancia, se encarga de medir los segmentos
        que el usuario ha dibujado en el mapa
    """
    def __init__( self, canvas, parent ):
        QgsMapTool.__init__( self, canvas )
        self.canvas=canvas

        self.qgsDist = QgsDistanceArea() # Esta clase realiza la medicin

        # Instanciar el dilogo medir
        self.dialogoMedir = DialogoMedirDistancia( parent )

        self.puntoInicialMove = None
        self.puntoFinalMove = None
        self.oldPuntoPantalla = None # Para validar cercana de 2 puntos consecutivos
        self.poligono = False
        self.bTerminada = False # Flag para el fin de la lnea

        # RubberBand dinmica
        self.rubberbandMove = QgsRubberBand( self.canvas, self.poligono )
        self.rubberbandMove.setColor( QColor( 255,121,48,255 ) )

        # RubberBand permanente
        self.rubberbandClick = QgsRubberBand( self.canvas, self.poligono )
        self.rubberbandClick.setColor( QColor( 255,121,48,255 ) )

    def canvasPressEvent( self, event ):        
        self.dialogoMedir.mostrar()    # Abrir el dilogo de la herramienta

        # Valida la cercana de 2 puntos consecutivos
        bValidacion = True
        if self.oldPuntoPantalla:
            pPunto = QPoint()
            pPunto = self.oldPuntoPantalla - event.pos()
            bValidacion = True if ( pPunto.manhattanLength() > 3 ) else False

        self.oldPuntoPantalla = QPoint( event.pos().x(), event.pos().y() )

        # Inicializar, dibujar o terminar la RBClick
        p = self.toMapCoordinates( QPoint( event.pos().x(), event.pos().y() ) )

        if event.button() == 1: # Botn Izquierdo
            if self.bTerminada == True: # Inicializar RubberBands
                self.rubberbandMove.reset( self.poligono )
                self.rubberbandClick.reset( self.poligono )
                self.bTerminada = False
                self.dialogoMedir.inicializar()

            if bValidacion: # Dibujar
                self.puntoInicialMove = p
                self.rubberbandClick.addPoint( p )

                # Agregar segmento al dilogo
                self.dialogoMedir.agregarMedidaSegmento()

        elif event.button() == 2: # Botn Derecho
            if self.bTerminada == False: # Terminar el dibujo
                self.rubberbandClick.addPoint( p )
                self.puntoInicialMove = None
                self.oldPuntoPantalla = None
                self.bTerminada = True

    def canvasMoveEvent( self, event ):
        # Inicializar dibujo de RBMove
        self.rubberbandMove.reset( self.poligono )
        self.rubberbandMove = QgsRubberBand( self.canvas, self.poligono )
        self.rubberbandMove.setColor( QColor( 255,121,48,255 ) )

        # Si no se ha terminado la lnea, continuar con la RBMove
        if self.puntoInicialMove and ( not self.bTerminada ):
            p = self.toMapCoordinates( QPoint( event.pos().x(), event.pos().y() ) )
            self.puntoFinalMove = p
            self.rubberbandMove.addPoint( self.puntoInicialMove )
            self.rubberbandMove.addPoint( self.puntoFinalMove )

            # Medir la longitud del segmento actual y pasarla al dilogo
            self.dialogoMedir.modificarMedidaSegmento( self.qgsDist.measureLine( self.puntoInicialMove, p ) )

    def deactivate( self ):
        self.rubberbandMove.reset( self.poligono )
        self.rubberbandClick.reset( self.poligono )

        self.puntoInicialMove = None
        self.oldPuntoPantalla = None

        self.dialogoMedir.inicializar()
        self.dialogoMedir.cerrar()

    def isZoomTool( self ):
        return False
